@@ -6,7 +6,7 @@ use 5.014;
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-our $VERSION = '1.04';
+our $VERSION = '1.05';
 
 use Carp qw(confess cluck);
 use DateTime;
@@ -34,44 +34,46 @@ sub new {
 		iris_base      => $opt{iris_base}
 		  // 'http://iris.noncd.db.de/iris-tts/timetable',
 		lookahead => $opt{lookahead} // ( 4 * 60 ),
+		main_cache   => $opt{main_cache},
+		rt_cache     => $opt{realtime_cache},
 		serializable => $opt{serializable},
-		station      => $opt{station},
 		user_agent   => $ua,
+		with_related => $opt{with_related},
 	};
 
 	bless( $self, $class );
 
 	$ua->env_proxy;
 
-	my $res_st = $ua->get( $self->{iris_base} . '/station/' . $opt{station} );
+	my ( $station, @related_stations ) = $self->get_station(
+		name      => $opt{station},
+		root      => 1,
+		recursive => $opt{with_related},
+	);
 
-	if ( $res_st->is_error ) {
-		$self->{errstr}
-		  = 'Failed to fetch station data: ' . $res_st->status_line;
-		return $self;
+	$self->{station}          = $station;
+	$self->{related_stations} = \@related_stations;
+
+	for my $ref (@related_stations) {
+		my $ref_status = Travel::Status::DE::IRIS->new(
+			datetime       => $self->{datetime},
+			developer_mode => $self->{developer_mode},
+			lookahead      => $self->{lookahead},
+			station        => $ref->{uic},
+			with_related   => 0,
+		);
+		if ( not $ref_status->errstr ) {
+			push( @{ $self->{results} }, $ref_status->results );
+		}
 	}
 
-	my $xml_st = XML::LibXML->load_xml( string => $res_st->decoded_content );
-
-	my $station_node = ( $xml_st->findnodes('//station') )[0];
-
-	# TODO parse 'meta' and maybe 'p' flags
-	# They're optional pointers to related platforms. For instance
-	# Berlin Hbf/BL -> Berlin HBf (tief), Berlin Hbf (S), ...
-
-	if ( not $station_node ) {
-		$self->{errstr}
-		  = "The station '$opt{station}' has no associated timetable";
+	if ( $self->{errstr} ) {
 		return $self;
 	}
 
-	$self->{station}      = $opt{station};
-	$self->{station_code} = $station_node->getAttribute('eva');
-	$self->{station_name} = $station_node->getAttribute('name');
-
 	my $dt_req = $self->{datetime}->clone;
 	for ( 1 .. 3 ) {
-		$self->get_timetable( $self->{station_code}, $dt_req );
+		$self->get_timetable( $self->{station}{uic}, $dt_req );
 		$dt_req->add( hours => 1 );
 	}
 
@@ -116,6 +118,108 @@ sub new {
 	return $self;
 }
 
+sub get_with_cache {
+	my ( $self, $cache, $url ) = @_;
+
+	if ( $self->{developer_mode} ) {
+		say "GET $url";
+	}
+
+	if ($cache) {
+		my $content = $cache->thaw($url);
+		if ($content) {
+			if ( $self->{developer_mode} ) {
+				say '  cache hit';
+			}
+			return ( ${$content}, undef );
+		}
+	}
+
+	if ( $self->{developer_mode} ) {
+		say '  cache miss';
+	}
+
+	my $ua  = $self->{user_agent};
+	my $res = $ua->get($url);
+
+	if ( $res->is_error ) {
+		return ( undef, $res->status_line );
+	}
+	my $content = $res->decoded_content;
+
+	if ($cache) {
+		$cache->freeze( $url, \$content );
+	}
+
+	return ( $content, undef );
+}
+
+sub get_station {
+	my ( $self, %opt ) = @_;
+
+	my @ret;
+	my $recursion_depth = $opt{recursion_depth} // 0;
+
+	my ( $raw, $err )
+	  = $self->get_with_cache( $self->{main_cache},
+		$self->{iris_base} . '/station/' . $opt{name} );
+	if ($err) {
+		$self->{errstr} = "Failed to fetch station data: $err";
+		return;
+	}
+
+	my $xml_st = XML::LibXML->load_xml( string => $raw );
+
+	my $station_node = ( $xml_st->findnodes('//station') )[0];
+
+	if ( not $station_node ) {
+		if ( $opt{root} ) {
+			$self->{errstr}
+			  = "The station '$opt{name}' has no associated timetable";
+		}
+		return;
+	}
+	if ( $recursion_depth > 5 ) {
+		cluck("Reached recursion depth $recursion_depth while tracking IDs");
+		return;
+	}
+
+	push(
+		@ret,
+		{
+			uic   => $station_node->getAttribute('eva'),
+			name  => $station_node->getAttribute('name'),
+			ds100 => $station_node->getAttribute('ds100'),
+		}
+	);
+
+	if ( $self->{developer_mode} ) {
+		printf( " -> %s (%s / %s)\n", @{ $ret[0] }{qw{name uic ds100}} );
+	}
+
+	if ( $opt{recursive} and $station_node->hasAttribute('meta') ) {
+		my @recursion_blacklist = @{ $opt{recursion_blacklist} // [] };
+		my @refs = split( m{ \| }x, $station_node->getAttribute('meta') );
+
+		push( @recursion_blacklist, $ret[0]{uic} );
+		for my $ref (@refs) {
+			if ( not( $ref ~~ \@recursion_blacklist ) ) {
+				push(
+					@ret,
+					$self->get_station(
+						name                => $ref,
+						recursive           => 1,
+						recursion_depth     => $recursion_depth + 1,
+						recursion_blacklist => \@recursion_blacklist,
+					)
+				);
+			}
+		}
+	}
+
+	return @ret;
+}
+
 sub add_result {
 	my ( $self, $station, $s ) = @_;
 
@@ -181,23 +285,17 @@ sub add_result {
 
 sub get_timetable {
 	my ( $self, $eva, $dt ) = @_;
-	my $ua = $self->{user_agent};
 
-	my $res = $ua->get(
+	my ( $raw, $err )
+	  = $self->get_with_cache( $self->{main_cache},
 		$dt->strftime( $self->{iris_base} . "/plan/${eva}/%y%m%d/%H" ) );
 
-	if ( $self->{developer_mode} ) {
-		say 'GET '
-		  . $dt->strftime( $self->{iris_base} . "/plan/${eva}/%y%m%d/%H" );
-	}
-
-	if ( $res->is_error ) {
-		$self->{warnstr}
-		  = 'Failed to fetch a schedule part: ' . $res->status_line;
+	if ($err) {
+		$self->{warnstr} = "Failed to fetch a schedule part: $err";
 		return $self;
 	}
 
-	my $xml = XML::LibXML->load_xml( string => $res->decoded_content );
+	my $xml = XML::LibXML->load_xml( string => $raw );
 
 	my $station = ( $xml->findnodes('/timetable') )[0]->getAttribute('station');
 
@@ -212,20 +310,18 @@ sub get_timetable {
 sub get_realtime {
 	my ($self) = @_;
 
-	my $eva = $self->{station_code};
-	my $res = $self->{user_agent}->get( $self->{iris_base} . "/fchg/${eva}" );
+	my $eva = $self->{station}{uic};
 
-	if ( $self->{developer_mode} ) {
-		say 'GET ' . $self->{iris_base} . "/fchg/${eva}";
-	}
+	my ( $raw, $err )
+	  = $self->get_with_cache( $self->{rt_cache},
+		$self->{iris_base} . "/fchg/${eva}" );
 
-	if ( $res->is_error ) {
-		$self->{warnstr}
-		  = 'Failed to fetch realtime data: ' . $res->status_line;
+	if ($err) {
+		$self->{warnstr} = "Failed to fetch realtime data: $err";
 		return $self;
 	}
 
-	my $xml = XML::LibXML->load_xml( string => $res->decoded_content );
+	my $xml = XML::LibXML->load_xml( string => $raw );
 
 	my $station = ( $xml->findnodes('/timetable') )[0]->getAttribute('station');
 
@@ -367,6 +463,18 @@ sub create_replacement_refs {
 	}
 }
 
+sub station {
+	my ($self) = @_;
+
+	return $self->{station};
+}
+
+sub related_stations {
+	my ($self) = @_;
+
+	return @{ $self->{related_stations} };
+}
+
 sub errstr {
 	my ($self) = @_;
 
@@ -412,7 +520,7 @@ Travel::Status::DE::IRIS - Interface to IRIS based web departure monitors.
 
 =head1 VERSION
 
-version 1.04
+version 1.05
 
 =head1 DESCRIPTION
 
